package fwadmin;

/*

  JFwadmin v 0.8

  Copyright (C) 1999-2000 Mickael Feledyn
  		     mickael.feledyn@cybergal.com

  This application is free software; you can redistribute it and/or
  modify it under the terms of the GNU Genereal Public License as
  published by the Free Software Foundation; either version 2 of
  the License, or (at your option) any later version.

  This application is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
  GNU General Public License for more details.

*/

import java.awt.*;
import javax.swing.*;
import java.awt.event.*;
import java.util.*;

public class AddOutputServiceDialog extends JDialog implements ActionListener, ItemListener
{
  IPAddress ipaddr;
  IPMask netmask;
  
  JPanel panel1 = new JPanel();
  JComboBox interfaceJComboBox = new JComboBox();
  JLabel interfaceJLabel = new JLabel();
  JRadioButton serviceJRadioButton = new JRadioButton();
  JRadioButton portRangeJRadioButton = new JRadioButton();
  JComboBox serviceJComboBox = null;
  JTextField portRangeLowTextField = new JTextField();
  JLabel lowPortRangeJLabel = new JLabel();
  JLabel portRangeHighJLabel = new JLabel();
  JTextField portRangeHighJTextField = new JTextField();
  JComboBox policyJComboBox = new JComboBox();
  JLabel policyJLabel = new JLabel();
  JButton okJButton = new JButton();
  JButton cancelJButton = new JButton();
  JLabel protocolJLabel = new JLabel();
  JComboBox protocolJComboBox = new JComboBox();
  JCheckBox specifyIPNetJCheckBox = new JCheckBox();
  private boolean okPressed = false;

  public AddOutputServiceDialog(Frame frame, String title, boolean modal)
  {
	super(frame, title, modal);
	try 
	{
	  jbInit();
	  for(Enumeration e = IPInterface.interfacesHashtable.elements(); e.hasMoreElements(); )
	  {
		IPInterface ipi = (IPInterface)e.nextElement();
		interfaceJComboBox.addItem(ipi.getName() + " (" + ipi.getKind() + ")");
	  }

	  ButtonGroup radioGroup = new ButtonGroup();
	  radioGroup.add(serviceJRadioButton);
	  radioGroup.add(portRangeJRadioButton);
	  serviceJRadioButton.addActionListener(this);
	  portRangeJRadioButton.addActionListener(this);
	  cancelJButton.addActionListener(this);
	  okJButton.addActionListener(this);
	  specifyIPNetJCheckBox.addItemListener(this);
	  specifyIPNetJCheckBox.setToolTipText("For specific needs");
	  portRangeLowTextField.setEnabled(false);
	  lowPortRangeJLabel.setEnabled(false);
	  portRangeHighJLabel.setEnabled(false);
	  portRangeHighJTextField.setEnabled(false);
	  policyJComboBox.addItem(DefaultPolicy.firewallAcceptPolicy);
	  policyJComboBox.addItem(DefaultPolicy.firewallDenyPolicy);
	  protocolJComboBox.addItem("tcp");
	  protocolJComboBox.addItem("udp");
	  protocolJComboBox.addItem("icmp");
	  protocolJComboBox.addItem("all");
	  pack();
	}
	catch(Exception ex)
	{
	  ex.printStackTrace();
	}
  }        
  public void actionPerformed(ActionEvent e)
  {
	if(e.getSource() == serviceJRadioButton)
	{
	  portRangeLowTextField.setEnabled(false);
	  lowPortRangeJLabel.setEnabled(false);
	  portRangeHighJLabel.setEnabled(false);
	  portRangeHighJTextField.setEnabled(false);
	  serviceJComboBox.setEnabled(true);
	}
	if(e.getSource() == portRangeJRadioButton)
	{
	  portRangeLowTextField.setEnabled(true);
	  lowPortRangeJLabel.setEnabled(true);
	  portRangeHighJLabel.setEnabled(true);
	  portRangeHighJTextField.setEnabled(true);
	  serviceJComboBox.setEnabled(false);
	}
	if(e.getSource() == cancelJButton)
	{
	  dispose();
	}
	if(e.getSource() == specifyIPNetJCheckBox)
	{
	  ipaddr = new IPAddress();
	  netmask = new IPMask();
	  IPNetDialog ipNetDlg = new IPNetDialog(this, "Specify source address", true, ipaddr, netmask, specifyIPNetJCheckBox);
	  ipNetDlg.setVisible(true);
	}
	if(e.getSource() == okJButton)
	{
		OutputRule or = new OutputRule();
		InputRule ir = new InputRule();
		try
		{
		  String selectedIF = (String) interfaceJComboBox.getSelectedItem();
		  StringTokenizer st = new StringTokenizer(selectedIF, " ");
		  String ifName = st.nextToken();
		  or.m_ruleType = "output";
		  or.m_sourceAddress = IPInterface.getInterface(ifName).getAddress();
		  or.m_sourceMask = new IPMask(0xFFFFFFFFL);
		  if (IPInterface.getInterface(ifName).getKind().equals(IPFWConstants.networkKindInternet))
		  {
			StringTokenizer st2 = new StringTokenizer(JFwadmin.getResource("anywhere_network"), "/");
			IPAddress ip = new IPAddress(0L);
			try{ip.fromString(st2.nextToken());}catch(Exception bad1){}
			IPMask mk = new IPMask(0L);
			try{ip.fromString(st2.nextToken());}catch(Exception bad2){}
			or.m_destinationAddress = ip;
			or.m_destinationMask = mk;
		  }
		  else
		  {
			or.m_destinationAddress = new IPAddress(IPInterface.getInterface(ifName).getAddress().getLong() & IPInterface.getInterface(ifName).getMask().getLong());
			or.m_destinationMask = new IPMask(IPInterface.getInterface(ifName).getMask().getLong());
		  }
		  if (specifyIPNetJCheckBox.isSelected())
		  {
			or.m_destinationAddress = ipaddr;
			or.m_destinationMask = netmask;
		  }
		  try
		  {
			StringTokenizer st2 = new StringTokenizer(JFwadmin.getResource("any_port"), ":");
			or.m_sourceLowPort = Integer.parseInt(st2.nextToken());
			or.m_sourceHighPort = Integer.parseInt(st2.nextToken());
		  }
		  catch(Exception e2)
		  {
			or.m_sourceLowPort = 0;
			or.m_sourceHighPort = 65535;
		  }	
		  if (serviceJRadioButton.isSelected())
		  {
			int portNumber = Linux22.getServicePortNumber((String)serviceJComboBox.getEditor().getItem(), (String)protocolJComboBox.getSelectedItem());
			if (portNumber == 0)
			{
			  JOptionPane.showMessageDialog(JFwadmin.getInstance(), "The selected service is not supported on " + (String)protocolJComboBox.getSelectedItem(), "Error", JOptionPane.ERROR_MESSAGE);
			  return;
			}
			or.m_destinationLowPort = portNumber;
			or.m_destinationHighPort = portNumber;
		  }
		  else
		  {
			or.m_destinationLowPort = Integer.parseInt(portRangeLowTextField.getText());
			or.m_destinationHighPort = Integer.parseInt(portRangeHighJTextField.getText());
		  }
		  or.m_interfaceName = ifName;
		  or.m_policy = (String) policyJComboBox.getSelectedItem();
		  or.m_protocol = Linux22.getProtocolConstant((String)protocolJComboBox.getSelectedItem());
		  ir.m_ruleType = "input";
		  ir.m_sourceAddress = or.m_destinationAddress;
		  ir.m_sourceMask = or.m_destinationMask;
		  ir.m_destinationAddress = or.m_sourceAddress;
		  ir.m_destinationMask = or.m_sourceMask;
		  ir.m_interfaceName = or.m_interfaceName;
		  ir.m_policy = or.m_policy;
		  ir.m_protocol = or.m_protocol;
		  ir.m_sourceLowPort = or.m_destinationLowPort;
		  ir.m_sourceHighPort = or.m_destinationHighPort;
		  ir.m_destinationLowPort = or.m_sourceLowPort;
		  ir.m_destinationHighPort = or.m_sourceHighPort;

		  ir.m_ruleDirtySectionOrdering = Linux22.DIRTY_ORDERING_SECTION_2_BASE + Linux22.m_ruleDirtyNumber;
		  or.m_ruleDirtySectionOrdering = Linux22.DIRTY_ORDERING_SECTION_2_BASE + Linux22.m_ruleDirtyNumber++;
	  }
	  catch(Exception e2)
	  {
		JOptionPane.showMessageDialog(this, "Please verify your inputs", "Errors in parameters", JOptionPane.WARNING_MESSAGE);
		return;
	  }
	  OutputService.outputServicesVector.addElement(new OutputService(ir, or, "not applied"));
	  InputRule.inputRulesVector.addElement(ir);
	  OutputRule.outputRulesVector.addElement(or);

	  okPressed = true;
	  dispose();
	}
  }                        
  public boolean getOK()
  {
	return okPressed;
  }  
public void itemStateChanged(ItemEvent e)
{
	if(e.getStateChange() == ItemEvent.SELECTED)
	if(isVisible())
	if(e.getSource() == specifyIPNetJCheckBox)
	{
	  ipaddr = new IPAddress();
	  netmask = new IPMask();
	  IPNetDialog ipNetDlg = new IPNetDialog(this, "Specify source address", true, ipaddr, netmask, specifyIPNetJCheckBox);
	  ipNetDlg.setVisible(true);
	}
}
  void jbInit() throws Exception
  {
	panel1.setLayout(null);
	interfaceJComboBox.setBounds(new Rectangle(21, 37, 205, 29));
	interfaceJLabel.setText("Interface");
	interfaceJLabel.setBounds(new Rectangle(18, 13, 139, 24));
	serviceJRadioButton.setSelected(true);
	serviceJRadioButton.setText("Service / port");
	serviceJRadioButton.setBounds(new Rectangle(20, 80, 172, 29));
	portRangeJRadioButton.setText("Ports range");
	portRangeJRadioButton.setBounds(new Rectangle(21, 164, 154, 24));
	serviceJComboBox = new JComboBox(Linux22.servicesVector);
	serviceJComboBox.setEditable(true);
	serviceJComboBox.setBounds(new Rectangle(43, 116, 196, 28));
	portRangeLowTextField.setBounds(new Rectangle(120, 196, 127, 26));
	lowPortRangeJLabel.setText("Low port");
	lowPortRangeJLabel.setBounds(new Rectangle(27, 198, 82, 24));
	portRangeHighJLabel.setText("High port");
	portRangeHighJLabel.setBounds(new Rectangle(265, 195, 79, 27));
	portRangeHighJTextField.setBounds(new Rectangle(348, 195, 123, 28));
	policyJComboBox.setBounds(new Rectangle(244, 37, 130, 29));
	policyJLabel.setText("Policy");
	policyJLabel.setBounds(new Rectangle(241, 15, 124, 23));
	okJButton.setText("OK");
	okJButton.setBounds(new Rectangle(119, 263, 113, 32));
	cancelJButton.setText("Cancel");
	cancelJButton.setBounds(new Rectangle(255, 263, 115, 34));
	specifyIPNetJCheckBox.setText("Specify destination network");
	specifyIPNetJCheckBox.setBounds(new Rectangle(285, 100, 202, 33));
	panel1.setMinimumSize(new Dimension(500, 310));
	panel1.setPreferredSize(new Dimension(500, 310));
	protocolJLabel.setText("Protocol");
	protocolJLabel.setBounds(new Rectangle(387, 16, 85, 22));
	protocolJComboBox.setBounds(new Rectangle(391, 37, 88, 29));
	getContentPane().add(panel1);
	panel1.add(interfaceJComboBox, null);
	panel1.add(interfaceJLabel, null);
	panel1.add(serviceJRadioButton, null);
	panel1.add(serviceJComboBox, null);
	panel1.add(portRangeJRadioButton, null);
	panel1.add(portRangeLowTextField, null);
	panel1.add(portRangeHighJTextField, null);
	panel1.add(okJButton, null);
	panel1.add(cancelJButton, null);
	panel1.add(protocolJComboBox, null);
	panel1.add(protocolJLabel, null);
	panel1.add(policyJComboBox, null);
	panel1.add(policyJLabel, null);
	panel1.add(lowPortRangeJLabel, null);
	panel1.add(portRangeHighJLabel, null);
	panel1.add(specifyIPNetJCheckBox, null);

	try
	{
		StringTokenizer st = new StringTokenizer(JFwadmin.getResource("any_port"), ":");
		portRangeLowTextField.setText(st.nextToken());
		portRangeHighJTextField.setText(st.nextToken());
	}
	catch(Exception e)
	{
		portRangeHighJTextField.setText("65535");
		portRangeLowTextField.setText("0");
	}	
  }              
  public void resetOK()
  {
	okPressed = false;
  }  
}
