package fwadmin;

/*

  JFwadmin v 0.8

  Copyright (C) 1999-2000 Mickael Feledyn
  		     mickael.feledyn@cybergal.com

  This application is free software; you can redistribute it and/or
  modify it under the terms of the GNU Genereal Public License as
  published by the Free Software Foundation; either version 2 of
  the License, or (at your option) any later version.

  This application is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
  GNU General Public License for more details.

*/

import java.util.Enumeration;

public class Service
{
  InputRule m_ir;
  OutputRule m_or;
  private String m_serviceComment;

  public Service(InputRule ir, OutputRule or)
  {
	m_ir = ir;
	m_or = or;
  }  
  public Service(InputRule ir, OutputRule or, String comment)
  {
	m_ir = ir;
	m_or = or;
	m_serviceComment = comment;
  }  
  public void addComment(String comment)
  {
	if (m_serviceComment == null)
	{
	  m_serviceComment = comment;
	  return;
	}
	m_serviceComment += "; " + comment;
  }  
  public static void addInputService(InputRule ir, OutputRule or)
  {
	InputService is = new InputService(ir, or);
	ir.m_affectedToService = true;
	or.m_affectedToService = true;
	InputService.inputServicesVector.addElement(is);

	// Try to set a good comment and ordering type on this service
	if ((ir.m_flags & Linux22.FLAG_LOGGING) > 0)
	{
	  is.addComment("logged");
	  ir.m_ruleDirtySectionOrdering += Linux22.DIRTY_ORDERING_SECTION_3_BASE;
	  or.m_ruleDirtySectionOrdering += Linux22.DIRTY_ORDERING_SECTION_3_BASE;
	}
	for (Enumeration e = IPInterface.interfacesHashtable.elements(); e.hasMoreElements(); )
	{
	  IPInterface ipi = (IPInterface) e.nextElement();
	  if (ir.m_policy.equals("DENY"))
	  if (! ipi.getName().equals(ir.m_interfaceName))
	  //if (! ipi.getKind().equals(IPFWConstants.networkKindInternet))
	  if (ir.sourceNetworkMatch(ipi))
	  {
		is.addComment("anti spooffing");
		if(ir.m_ruleDirtySectionOrdering >= Linux22.DIRTY_ORDERING_SECTION_3_BASE) ir.m_ruleDirtySectionOrdering -= Linux22.DIRTY_ORDERING_SECTION_3_BASE;
		if(or.m_ruleDirtySectionOrdering >= Linux22.DIRTY_ORDERING_SECTION_3_BASE) or.m_ruleDirtySectionOrdering -= Linux22.DIRTY_ORDERING_SECTION_3_BASE;
		ir.m_ruleDirtySectionOrdering -= Linux22.DIRTY_ORDERING_SECTION_2_BASE;
		or.m_ruleDirtySectionOrdering -= Linux22.DIRTY_ORDERING_SECTION_2_BASE;
		ir.m_ruleDirtySectionOrdering += Linux22.DIRTY_ORDERING_SECTION_1_BASE;
		or.m_ruleDirtySectionOrdering += Linux22.DIRTY_ORDERING_SECTION_1_BASE;
	  }
	}
	if ((ir.m_protocol == Linux22.IPPROTO_ICMP) ||
		((ir.m_destinationLowPort == 0) && (ir.m_destinationHighPort == 65535) && (ir.m_sourceLowPort == 0) && (ir.m_sourceHighPort == 65535) && ir.portsMatch(or)))
	{
	  is.addComment("Is also output rule");
	}
  }                                  
  public static void addOutputService(InputRule ir, OutputRule or)
  {
	OutputService os = new OutputService(ir, or);
	ir.m_affectedToService = true;
	or.m_affectedToService = true;
	OutputService.outputServicesVector.addElement(os);

	if ((or.m_flags & Linux22.FLAG_LOGGING) > 0)
	{
	  os.addComment("logged");
	  ir.m_ruleDirtySectionOrdering += Linux22.DIRTY_ORDERING_SECTION_3_BASE;
	  or.m_ruleDirtySectionOrdering += Linux22.DIRTY_ORDERING_SECTION_3_BASE;
	}
  }          
  public String getComment()
  {
	if (m_serviceComment == null) return "";
	return m_serviceComment;
  }  
  public static void initServices()
  {
	  for (int i=0; i < InputRule.inputRulesVector.size(); i++)
	  {
	InputRule ir = (InputRule) InputRule.inputRulesVector.elementAt(i);
	  for (int j=0; j < OutputRule.outputRulesVector.size(); j++)
	  {
			OutputRule or = (OutputRule) OutputRule.outputRulesVector.elementAt(j);

			if (ir.m_interfaceName.equals(or.m_interfaceName))
			if (ir.m_protocol == or.m_protocol)
			if (ir.portsMatch(or))
			if (ir.m_policy.equals(or.m_policy))
	  if (ir.m_sourceAddress.equals(or.m_destinationAddress))
	  if (ir.m_destinationAddress.equals(or.m_sourceAddress))
	  if (ir.m_sourceMask.equals(or.m_destinationMask))
	  if (ir.m_destinationMask.equals(or.m_sourceMask))
			{
		if (or.m_affectedToService)
		{
		  //System.out.println("Rule matches taken rule : ");
		  //System.out.println(ir.toString());
		  //System.out.println(or.toString());
		}
		else
		// Rules match
		if((ir.isSourcePort(1024)) && (ir.isSourcePort(65535)))
		{
		  // Hope this is an input service
		  addInputService(ir, or);
		}
		else
		{
		  // Hope this is an output service
		  addOutputService(ir, or);
		}
			}
	}
	}

	// Now find unmatched rules, and put them in miscellaneous rules
	  for (int i=0; i < InputRule.inputRulesVector.size(); i++)
	  {
		  InputRule ir = (InputRule) InputRule.inputRulesVector.elementAt(i);
	  if (! ir.m_affectedToService)
	  {
		//System.out.println("Unmatched input rule : " + ir.toString());
		InputOutputRule.miscRulesVector.addElement(ir);
	  }
	}
	  for (int i=0; i < OutputRule.outputRulesVector.size(); i++)
	  {
			OutputRule or = (OutputRule) OutputRule.outputRulesVector.elementAt(i);
	  if (! or.m_affectedToService)
	  {
		//System.out.println("Unmatched output rule : " + or.toString());
		InputOutputRule.miscRulesVector.addElement(or);
	  }
	}
  }        
}
